clear all

%------------------------------------------------------------------------
% This code uses a wavelet-Galerkin approach to solve for tip displacements
% of an axially vibrating rod (fixed on the left, and force applied to the
% free end on the right), where the tip can come in contact with a
% rigid wall.


%------------------------------------------------------------------------
% If the forcing frequency is 150Hz, the time-stepping solution will be
% plotted against the wavelet-Galerking solution at the end

f_forcing = 150;            % forcing frequency in Hz


%------------------------------------------------------------------------
J = 5;          % level of the scaling function to use
DB_ord = 6;     % order of the Daubechies scaling function (must be an even number 6 or greater)

% run a check to ensure there will be enough discrete points to use the
% order and level selected
if 2^J < (2*(DB_ord-2)+1)
    keyboard        % need more points, increase J
end


%------------------------------------------------------------------------
% Parameters for the axially vibrating rod

rho = 2700;         % density
E = 70e9;           % elastic modulus
A = 5e-2*5e-2;      % cross-sectional area
L_total = 1;        % length of the rod
dampFact = 1e-4;    % damping factor

F = 250e3;                  % force applied to the tip of the rod
omega = (2*pi)*f_forcing;   % forcing frequency in rad/s

nElem = 25;                 % number of elements used to define rod
L = L_total/nElem;          % element length
xNodal = (L:L:L_total)';    % unconstrained node locations

gap2spring = 1.0e-3;        % the gap between the end of the rod and the "wall"
penaltyForce = 1e8;         % penalty term for the "wall" spring
expTerm = 50;               % exponential term for the "wall" spring

tMax = 2*pi/omega;                      % max time value
t = (0:tMax/2^J:(tMax*(1-1/2^J)))';     % create time vector
dt = t(2)-t(1);                         % time-step between temporal points


%------------------------------------------------------------------------
% Create the finite element model
stiff = zeros(nElem+1);
mass = zeros(nElem+1);

% form the global mass and stiffness matrices
for JJ = 1:nElem;
    current_dof = JJ + (0:1);
    stiff(current_dof,current_dof) = stiff(current_dof,current_dof) + E*A/L*[1 -1;-1 1];
    mass(current_dof,current_dof) = mass(current_dof,current_dof) + rho*A*L/6*[2 1;1 2];
end

% Constrain the left side of the model (i.e. node 1)
stiff(1,:) = [];
stiff(:,1) = [];
mass(1,:) = [];
mass(:,1) = [];

%---------------------------------------------------------------
% Construct the 2-term connection coefficients (i.e. inner product terms)
% using Daubechies scaling functions as required for the Galerking approach

[phi0,pIndex]=func_calc_phi(DB_ord,0);
[phi1,pIndex]=func_calc_phi(DB_ord,1);
[GammaVec0] = func_calculate_2termCC(DB_ord,0);
[GammaVec1] = func_calculate_2termCC(DB_ord,1);
[GammaVec2] = func_calculate_2termCC(DB_ord,2);

% Compute a typical row for the "Galerkin Matrices" (GM) used below.  Since
% the solution is assumed periodic, these GM matrices can be created using
% a circular shift to limit the processing requirements.
typ0 = [GammaVec0(find(GammaVec0(:,1) == (DB_ord-1)),3);zeros(2^J - (2*(DB_ord-2)+1),1)]';
typ0 = circshift(typ0,[0 -(DB_ord-2)]);
typ1 = [GammaVec1(find(GammaVec1(:,1) == (DB_ord-1)),3);zeros(2^J - (2*(DB_ord-2)+1),1)]';
typ1 = circshift(typ1,[0 -(DB_ord-2)]);
typ2 = [GammaVec2(find(GammaVec2(:,1) == (DB_ord-1)),3);zeros(2^J - (2*(DB_ord-2)+1),1)]';
typ2 = circshift(typ2,[0 -(DB_ord-2)]);

for II = 1:2^J
    GM0(II,:) = circshift(typ0,[0 (II-1)]);
    GM1(II,:) = (1/dt)*circshift(typ1,[0 (II-1)]);
    GM2(II,:) = (1/dt^2)*circshift(typ2,[0 (II-1)]);
end
clear typ* GammaVec*


%--------------------------------------------------------------------------
% compile the Galerkin matrices, test function (u) indexed with k,
% weighting function (v) indexed with j

% note the damping matrix is assumed to be beta-damped (i.e. proportional
% to the stiffness matrix) by the parameter dampFact
for II = 1:size(mass,1);
    for JJ = 1:size(mass,2);
        row_position = (II-1)*size(GM0,1) + (1:size(GM0,1));
        col_position = (JJ-1)*size(GM0,1) + (1:size(GM0,1));
        LHS_m(row_position,col_position) = mass(II,JJ)*GM2;
        LHS_d(row_position,col_position) = dampFact*stiff(II,JJ)*GM1;
        LHS_s(row_position,col_position) = stiff(II,JJ)*GM0;
    end
end

% compute the left-hand side (LHS) of the Galerkin equation
% LHS*SFC = RHS, where SFC are the scaling function coefficients to be
% found.  Note, integration by parts is not used in this formulation.
LHS = LHS_m + LHS_d + LHS_s;

%--------------------------------------------------------------------------
% Condensation - subscript k is to keep, c is to condense
% The space-time Galerkin Matrices are condensed so only the node at the
% tip of the vibrating bar is kept active for all time discretization.
% This greatly improves the non-linear solving efficiency and accuracy.

Acc = LHS(1:(nElem-1)*2^J,1:(nElem-1)*2^J);
Ack = LHS(1:(nElem-1)*2^J,((nElem-1)*2^J + 1):end);
Akc = LHS(((nElem-1)*2^J + 1):end,1:(nElem-1)*2^J);
Akk = LHS(((nElem-1)*2^J + 1):end,((nElem-1)*2^J + 1):end);
clear LHS

LHS_condensed = (Akk - Akc/Acc*Ack);

% compute the RHS from the forcing term
F_end = F*cos(omega*t);
typ = zeros(1,length(F_end));
typ(end-(DB_ord-3)-1:end-1) = flipud(phi0);     % offset to make the interpolation easier
for II = 2^J:-1:1
    phiDisp(II,:) = circshift(typ,[0 (II-2^J)]);
end

RHS_condensed = phiDisp\F_end;

%-------------------------------------------------------------------------
% compute the Phi terms required to transform from the scaling function
% domain back to the spatial domain.  This transform ensures the temporal
% solution will have 256 discrete points to match up with the time-stepping
% solution.
cd Phi_Interp
[phi_interp0] = func_phi_interp(DB_ord,(8-J),0);
[phi_interp1] = func_phi_interp(DB_ord,(8-J),1);
cd ..

% transformation matrix for displacement terms
typ = zeros(2^8,1);
typ(1:length(phi_interp0)) = phi_interp0;
for II = 1:2^J
    phiDisp_full(:,II) = circshift(typ,(2^(8-J))*(II-1));
end
clear typ

% transformation matrix for velocity terms
typ = zeros(2^8,1);
typ(1:length(phi_interp1)) = phi_interp1;

for II = 1:2^J
    phiVel_full(:,II) = circshift(typ,(2^(8-J))*(II-1));
end

%--------------------------------------------------------------------------
% Set up some options for the fsolve algorithm
opts = [];
opts = optimset('fsolve');
opts.Display = 'off';
opts.PlotFcns = {@optimplotx,@optimplotfval};
opts.MaxIter = 1000;
opts.MaxFunEvals = 5000;
opts.TolX = 1e-10;
opts.TolFun = 1e-10;
opts.Algorithm = 'trust-region-reflective';
%opts.Algorithm = 'levenberg-marquardt';
%opts.Algorithm = 'trust-region-dogleg';

u0 = zeros(size(LHS_condensed,1),1);        % initial guess at solution
[u_mk,feval,exitflag] = fsolve(@(u_mk)func_exponential_MDOF_condensed(u_mk,phiDisp,phiDisp,LHS_condensed,RHS_condensed,gap2spring,penaltyForce,expTerm),u0,opts);

%--------------------------------------------------------------------------
u_temp = [-Acc\Ack*u_mk;u_mk];      % expand the condensed solution

% convert from the scaling function participation coefficients (u_temp) to
% spatial solution for displacement (y_all) and velocity (v_all)
for II = 1:nElem
    indexSpan = (((II-1)*2^J + 1):((II-1)*2^J + 2^J))';
    y_all(II,:) = phiDisp_full*(u_temp(indexSpan));
    v_all(II,:) = 1/dt*phiVel_full*(u_temp(indexSpan));
    clear indexSpan
end
y_tip_WG = y_all(end,:);        % the displacement of the tip over time
y_tip_WG = y_tip_WG(:);

% compute the equivalent penalty force at the wall
wallForce = max(penaltyForce*(exp(expTerm*(y_all(end,:)-gap2spring))-1),0);

%--------------------------------------------------------------------------
% load some time-stepping solutions if the forcing frequency is 150Hz

t_plot = (0:1/2^8:(1-1/2^8))';

if f_forcing == 150;
    load timeStep_150Hz_full
    y_tip_TS = u_end;
end

%--------------------------------------------------------------------------
% plot the results
figure(101)
clf
h(1) = plot(t_plot,y_tip_WG*1000,'g');
legend('W-G solution')
xlabel('Normalized Time');
ylabel('Amplitude (mm)');
if (f_forcing == 150)
    hold on
    h(2) = plot(t_plot,y_tip_TS*1000,'b');
    
    legend('W-G solution','Time-Stepping solution')
end


