function [resultVec] = func_calculate_3termCC(L,m,n)
% This program calculates the orthogonal wavelet 3-term connection
% coefficients using the improved algorithm developed by S. Jones (2011).

% Input values which can be modified include
% L - the wavelet order (must be an even number)
% m - the derivative of the first scaling function term
% n - the derivative of the second scaling function term

% The computed coefficients are stored in the matrix resultVec
% where column 1 = x value, column 2 = j value, column 3 = k value
% and the final column are the 3-term connection coefficients

% Note m,n <= (L/2-1) !!!

%--------------------------------------------------------------
% Define the allowable x, j, and k values in vector form to be used below
xVec = (1:(L-1))';
jVec = (-(L-2):(L-2))';
kVec = (-(L-2):(L-2))';

%--------------------------------------------------------------
% import the wavelet filter coefficients
[p] = func_calc_filter_coefs(L);

%--------------------------------------------------------------
% compute the indexing for all the unknowns to be found for x=1,2,...,(L-1)
% the matrix admis_index has three columns: [x index, j index, k index]
[admis_index] = func_calc_admissible_index(L,jVec,kVec);

%--------------------------------------------------------------
% compile the Scaling Equation matrix as per Eqn (113) Chen et al.
[S_Mat] = func_calc_scaling_equations(admis_index,L,p);

%--------------------------------------------------------------
% find the eigenvalues corresponding to 2^(1-m-n).  The associated
% eigenvectors provide the solution space.  The unique solution will be
% found in this space using the moment equations below.
% Note the error in Eqn (114), should be 2^(1-m-n)v = Sv

EoI = 2^(1-m-n);
[v,d] = eig(S_Mat);
d = diag(d);
temp = find((d>0.9999*EoI)&(d<1.0001*EoI));     %account for numerical error
eigVec = v(:,temp);         % contains the vectors defining the solution space
clear temp v d

% Check to make sure the solution space is the correct size calculated
% using Eqn (120) and condition under (116)
if (m+n)<=(L/2)
    II = (1:(m+n))';
    expected_solution_space_size = (m+n+1) + sum(II);
    clear II
else
    II = ((L/2+1):(m+n))';
    expected_solution_space_size = (m+n+1) + (sum(3*L/2-2*II) + (L+2)*L/8);
    clear II
end
if size(eigVec,2) ~= expected_solution_space_size
    keyboard        % If this triggers the eigenvalues were found incorrectly
end

%--------------------------------------------------------------
% Create a connectivity matrix containing all the solution space vectors
% taking into account the backfill due to Eqn (112)
[soluSpace] = func_calc_solution_space(eigVec,admis_index,xVec,jVec,kVec,L);

%--------------------------------------------------------------
% Compile all the moment equations for x=1,2,...,(L-1)
SSS = expected_solution_space_size;
for II = 1:SSS
    workingVec = [soluSpace(:,1:3) soluSpace(:,(3+II))];
    [temp_LHS_117,temp_RHS_117] = func_calc_MomentEqns117(workingVec,xVec,jVec,kVec,L,m,n);
    [temp_LHS_118,temp_RHS_118] = func_calc_MomentEqns118(workingVec,xVec,jVec,kVec,L,m,n);
    
    if II == 1
        ME_LHS_117 = temp_LHS_117;
        ME_RHS_117 = temp_RHS_117;
        ME_LHS_118 = temp_LHS_118;
        ME_RHS_118 = temp_RHS_118;
    else
        ME_LHS_117 = [ME_LHS_117 temp_LHS_117(:,end)];
        ME_LHS_118 = [ME_LHS_118 temp_LHS_118(:,end)];
        % don't need RHS since it doen't change
    end
    clear temp_* workingVec
end

% Get rid of any moment equations where the LHS and RHS are equal to 0
% These equations aren't need because 0=0 isn't too helpful!
for II = size(ME_LHS_117,1):-1:1
    if (sum(abs(ME_LHS_117(II,3:end))) == 0) & (ME_RHS_117(II,3) == 0)
        ME_LHS_117(II,:) = [];
        ME_RHS_117(II,:) = [];
    end
    if (sum(abs(ME_LHS_118(II,3:end))) == 0) & (ME_RHS_118(II,3) == 0)
        ME_LHS_118(II,:) = [];
        ME_RHS_118(II,:) = [];
    end
end

% Concatinate the LHS and RHS for Eqn (117) and (118) into respective
% matrices.  Also get rid of the first two columns (indexing) so the
% psuedoinverse can be used.
ME_LHS = [ME_LHS_117(:,3:end); ME_LHS_118(:,3:end)];
ME_RHS = [ME_RHS_117(:,3); ME_RHS_118(:,3)];
clear ME_LHS_117 ME_RHS_117 ME_LHS_118 ME_RHS_118

%--------------------------------------------------------------
% Solution step

% Determine the "best-fit" scaling coefficients for each of the solution
% space vectors required to adhere to the moment equations.
coefs = pinv(ME_LHS)*ME_RHS;
clear ME_LHS ME_RHS

% Calculate the unique vector in the solution space using these coeffs
w = 0;
for II = 1:SSS
    w = w + coefs(II)*soluSpace(:,3+II);
end

% Numerical error may cause some imaginary parts to show up, thus just take
% the real part.  The imaginary part will be negligably small
resultVec = [soluSpace(:,1:3) real(w)];

